from datetime import datetime
import logging
from logging.handlers import TimedRotatingFileHandler
import os
from pathlib import Path

import config

def init_logging():
    formatter = logging.Formatter(
        '%(levelname)1.1s %(asctime)s %(module)15s:%(lineno)03d %(funcName)15s) %(message)s',
        datefmt='%H:%M:%S')
    # formatter = logging.Formatter(
    #     '[%(levelname)1.1s %(process)5d-%(thread)11d %(asctime)s %(module)15s:%(lineno)03d %(funcName)15s] %(message)s',
    #     datefmt='%H:%M:%S')
    
    dir = "../_log"
    Path(dir).mkdir(parents=True, exist_ok=True)
    path = os.path.join(dir, "abdi.log")
    file_handler = TimedRotatingFileHandler(path, when="d")
    file_handler.setFormatter(formatter)

    console_handler = logging.StreamHandler()
    console_handler.setFormatter(formatter)

    logger = logging.getLogger()
    logger.addHandler(file_handler)
    logger.addHandler(console_handler)
    logger.setLevel(config.log_level)

def print_log(msg, ex=None):
    print("[%s] %s" % (str(datetime.now())[5:-3], msg))
    if ex:
        print(ex)



from holon.HolonicAgent import HolonicAgent
from guide.hearing.Microphone import Microphone
from guide.hearing.BackgroundDenoising import BackgroundDenoising

class Hearing(HolonicAgent) :
    def __init__(self):
        super().__init__()
        self.head_agents.append(Microphone())
        self.body_agents.append(BackgroundDenoising())



from holon.HolonicAgent import HolonicAgent
from guide.voice.Speaker import Speaker
from guide.voice.ToneProcessing import ToneProcessing

class Voice(HolonicAgent) :
    def __init__(self):
        super().__init__()
        self.head_agents.append(Speaker())
        self.body_agents.append(ToneProcessing())



from holon.HolonicAgent import HolonicAgent
from guide.visual.Camera import Camera
from guide.visual.ImagePreprocessing import ImagePreprocessing

class Visual(HolonicAgent) :
    def __init__(self):
        super().__init__()
        self.head_agents.append(Camera())
        self.body_agents.append(ImagePreprocessing())



import logging

from guide.GuideMain import GuideMain
import Helper

if __name__ == '__main__':
    Helper.init_logging()
    logging.info('***** Main start *****')

    a = GuideMain()
    a.start()



from guide.navi.walk.KanbanDetect import KanbanDetect
from guide.navi.walk.RoadDetect import RoadDetect
from holon.HolonicAgent import HolonicAgent

class WalkGuide(HolonicAgent):
    def __init__(self):
        super().__init__()
        self.body_agents.append(KanbanDetect())
        self.body_agents.append(RoadDetect())





from guide.navi.VisualInput import VisualInput
from holon.HolonicAgent import HolonicAgent
from guide.navi.RouteFind import RouteFind
from guide.navi.walk.WalkGuide import WalkGuide

class NaviSystem(HolonicAgent) :
    def __init__(self):
        super().__init__()
        self.head_agents.append(VisualInput())
        self.body_agents.append(WalkGuide())
        self.body_agents.append(RouteFind())





import logging
import time

from holon.HolonicAgent import HolonicAgent
from guide.visual.Visual import Visual
from guide.hearing.Hearing import Hearing
from guide.voice.Voice import Voice
from guide.navi.NaviSystem import NaviSystem
from guide.dialog.DialogSystem import DialogSystem

class GuideMain(HolonicAgent) :
    def __init__(self):
        super().__init__()
        self.body_agents.append(NaviSystem())
        self.body_agents.append(DialogSystem())
        self.head_agents.append(Visual())
        self.head_agents.append(Hearing())
        self.head_agents.append(Voice())

    def _run(self):
        logging.info(f"Run GuideMain")
        time.sleep(2)


        


import logging
import inspect
from multiprocessing import Process
import os
import signal
import sys
import threading
import time 

import paho.mqtt.client as mqtt

currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0, parentdir) 
import config
import Helper
from core.Agent import Agent
from holon.Blackboard import Blackboard
from holon.HolonicDesire import HolonicDesire
from holon.HolonicIntention import HolonicIntention

class HolonicAgent(Agent) :
    def __init__(self, b:Blackboard=None, d:HolonicDesire=None, i: HolonicIntention=None):
        b = b or Blackboard()
        d = d or HolonicDesire()
        i = i or HolonicIntention()
        super().__init__(b, d, i)
        self.head_agents = []
        self.body_agents = []
        self.mqtt_client = None
        self.run_interval_seconds = 1
        self.is_running = False

    def _on_connect(self, client, userdata, flags, rc):
        logging.info(f"MQTT is connected with result code {str(rc)}")
        client.subscribe("echo")
        client.subscribe("terminate")

    def _on_message(self, client, db, msg):
        data = msg.payload.decode('utf-8', 'ignore')
        logging.debug("topic: %s, data: %s" % (msg.topic, data))
        # if data:
        #     try:
        #         data = json.loads(data)
        #     except:
        #         write_log("[ERROR] json.loads(data)")
        #         topic = ""
        #         data = {}

        if "terminate" == msg.topic:
            self._terminate_lock.set()
        # elif "awakable" == msg.topic:
        #     self.awakable = "on" == data["status"]
        # elif "indoor" == msg.topic:
        #     self.indoor = "on" == data["status"]
        # elif "kanban_indoor" == msg.topic:
        #     Information.set_indoor_kanbans(data)
        # elif "arrived" == msg.topic:
        #     self.arrived = "on" == data["status"]
        # elif "echo" == msg.topic:
        #     write_log("An echo got.")
    
    def _run_begin(self):
        Helper.init_logging()
        logging.info(f"_run_begin: {self.__class__.__name__}")

        self._terminate_lock = threading.Event()
        self._start_mqtt()
        threading.Thread(target=self.__interval_loop).start()
        
        def signal_handler(signal, frame):
            self._terminate_lock.set()
        signal.signal(signal.SIGINT, signal_handler)
        
        self.is_running = True
        # Helper.write_log("_run_begin 2")

    def __interval_loop(self):
        while not self._terminate_lock.is_set():
            self._run_interval()
            time.sleep(self.run_interval_seconds)

    def _run_interval(self):
        pass
        # Helper.write_log("_run_interval")

    def _running(self):
        logging.debug("running")

    def _run(self):
        self._run_begin()
        self._running()
        self._run_end()
    
    def _run_end(self):
        logging.warn(f"_run_end: {self.__class__.__name__}")
        self.is_running = False
        self._terminate_lock.wait()
        self._stop_mqtt()

    def _start_mqtt(self):
        logging.info("Starting...")
        self.mqtt_client = mqtt.Client()
        self.mqtt_client.on_connect = self._on_connect
        self.mqtt_client.on_message = self._on_message
        if config.mqtt_username:
            self.mqtt_client.username_pw_set(config.mqtt_username, config.mqtt_password)
        self.mqtt_client.connect(config.mqtt_address, config.mqtt_port, config.mqtt_keepalive)
        self.mqtt_client.loop_start()

    def _stop_mqtt(self):
        logging.warn("_stop_mqtt")
        self.mqtt_client.disconnect()
        self.mqtt_client.loop_stop()
        
    def start(self):
        p = Process(target=self._run)
        p.start()

        for a in self.head_agents:
            logging.info(f"head: {a.__module__} start")
            a.start()
        for a in self.body_agents:
            logging.info(f"body: {a.__module__} start")
            a.start()
        






from holon.HolonicAgent import HolonicAgent
from guide.dialog.Nlu import Nlu
from guide.dialog.AudioInput import AudioInput
from guide.dialog.AudioOutput import AudioOutput

class DialogSystem(HolonicAgent) :
    def __init__(self):
        super().__init__()
        self.head_agents.append(AudioOutput())
        self.head_agents.append(AudioInput())
        self.body_agents.append(Nlu())
        




from setuptools import setup, find_packages

setup(
    name='abdi',
    version='1.0.0',
    author='Shiu Ming Fang',
    author_email='avatar.xu@gmail.com',
    description='Agent BDI framework',
    packages=find_packages(),
)